#include <Kafka_Provider.h>

using namespace inaf::oasbo::Providers;

KafkaProviderBuilder::KafkaProviderBuilder() {
	this->reset();
}
KafkaProviderBuilder::KafkaProviderBuilder(std::string ip, int port, std::string topic) {
	this->provider = new KafkaProvider(ip, port, topic);

}
KafkaProviderBuilder::~KafkaProviderBuilder() {
	delete provider;
}

void KafkaProviderBuilder::reset() {
	this->provider = new KafkaProvider();
}

KafkaProviderBuilder* KafkaProviderBuilder::configFrom(Configurators::BaseConfigurator &conf) {
	conf.readConfigFromSource(config_target);
	std::map<std::string, std::string> params = conf.getConfig();

	std::string key = config_target+"_"+ip_key;
	if (params.count(key) > 0)
		provider->brokerIp = params[key];

	key = config_target+"_"+port_key;
	if (params.count(key) > 0)
		provider->brokerPort = std::stoi(params[key]);

	key = config_target+"_"+topic_key;
	if (params.count(key) > 0)
		provider->setDest(params[key]);
	return this;
}

KafkaProviderBuilder* KafkaProviderBuilder::setIp(std::string ip) {
	provider->brokerIp = ip;
	return this;
}

KafkaProviderBuilder* KafkaProviderBuilder::setPort(int port) {
	provider->brokerPort = port;
	return this;
}

KafkaProviderBuilder* KafkaProviderBuilder::setTopic(std::string topic) {
	provider->setDest(topic);
	return this;
}

KafkaProvider* KafkaProviderBuilder::getProvider() {
	KafkaProvider *result = this->provider;
	this->reset();
	return result;
}
